/**
 *#########################################################################
 * MetaData.java - part of the demo-client for Greenstone 3, of the 
 * Greenstone digital library suite from the New Zealand Digital Library 
 * Project at the  * University of Waikato, New Zealand.
 * <BR><BR>
 * Copyright (C) 2008 New Zealand Digital Library Project
 * <BR><BR>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * <BR><BR>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *########################################################################
 */

package org.greenstone.gs3client.data;

import org.greenstone.gsdl3.util.GSXML;
import org.w3c.dom.Element;

/** 
 * Stores the name, value and value of body textnode of a simple &lt;metadata&gt;
 * element. &lt;metadata name="str" value="str"&gt;bodyvalue&lt;/metadata&gt; 
 * @author ak19
*/
public class MetaData {
	/* Data fields in a &lt;metadata&gt; element */
	/** Name attribute of a &lt;metadata&gt; */
	public final String name;
	/** Value attribute of a &lt;metadata&gt; - this may or may not be there */
	public final String value; 
	/** Text node content of a &lt;metadata&gt; - the textnode may be empty, in
	 * which case this will be set to "" */
	public final String bodyText;
	
	/** Constructor that creates a MetaData object from a 
	 * &lt;metadata&gt;&lt;/metadata&gt; element by setting its members fields 
	 * using info extracted from the element. 
	 * @param metadataTag is the &lt;metadata&gt; element whose data is used
	 * to construct this MetaData object */
	public MetaData(Element metadataTag){
		// we'd definitely have a name attribute, but to be safe:
		this.name = metadataTag.hasAttribute(GSXML.NAME_ATT) ?
				metadataTag.getAttribute(GSXML.NAME_ATT) : "";
		//value need to be present		
		this.value = metadataTag.hasAttribute(GSXML.VALUE_ATT) ?
				metadataTag.getAttribute(GSXML.VALUE_ATT) : "";
				
		// Look for any values nested inside <metadata></metadata>:
		if(metadataTag.hasChildNodes()) 
					// && metadataTag.getFirstChild() != null
			// get its textnode's text:
			this.bodyText = metadataTag.getFirstChild().getNodeValue(); 
		else this.bodyText = "";
	}
	
	/** @return the name of this Metadata object. Useful when adding MetaData
	 * objects to a JList or JCombobox */
	public String toString() { return name; }
	
	/** @return a String displaying the contents of this MetaData object. 
	 * Useful for debugging purposes.*/
	public String show() {
		StringBuffer buf = new StringBuffer("name: " + name);
		if(!value.equals("")) 
			buf.append(" -value: " + value);
		if(!bodyText.equals("")) 
			buf.append(" -bodyText: " + bodyText);
		return buf.toString();
	}
}