/**
 *#########################################################################
 * Pair.java - part of the demo-client for Greenstone 3, of the 
 * Greenstone digital library suite from the New Zealand Digital Library 
 * Project at the  * University of Waikato, New Zealand.
 * <BR><BR>
 * Copyright (C) 2008 New Zealand Digital Library Project
 * <BR><BR>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * <BR><BR>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *########################################################################
 */

package org.greenstone.gs3client.data;

import java.util.Vector;

/** Pair is a simple utility class to represent two associated String values.
 * For instance, a Metadata (name, value) pair to be displayed in either  
 * SearchResultsDisplay or BrowseDisplay's metanames and metavalues
 * JLists. (These two classes can be found in org.greenstone.gs3client.)
 * A Comparator--like the one provided by org.greenstone.gs3client.Displays' 
 * inner class MetadataComparator--is necessary to ensure that the "natural  
 * ordering" of any List of metadata Pairs will be as required. This class
 * does not implement Comparable so that it may be used for other purposes
 * than storing ordered Metadata pairs, which is why using an external
 * Comparator to provide the sorting facility is more useful. 
 * @author ak19 */
public class Pair {
	/** first item of the Pair */
	public final String first;
	/** second item of the Pair */
	public final String second;
	
	/** Constructor that creates a Pair from two String values. 
	 * @param a - the value for the first item of the Pair 
	 * @param b - the value for the second item of the Pair. */
	public Pair(String a, String b) {
		first = a;
		second = b;
	}
	
	/** Given a Vector of Pairs, returns an array of all the first
	 * values of each Pair.
	 * @return all the first fields in a Vector of pairs as an array. */
	public static String[] getFirst(Vector pairs) {
		String[] firsts = new String[pairs.size()];
		for(int i = 0; i < pairs.size(); i++) {
			Pair p = (Pair)pairs.get(i);
			firsts[i] = p.first;
		}
		return firsts;
	}
	
	/** Given a Vector of Pairs, returns an array of all the second
	 * values of each Pair.
	 * @return all the second fields in a Vector of pairs as an array. */
	public static String[] getSecond(Vector pairs) {
		String[] seconds = new String[pairs.size()];
		for(int i = 0; i < pairs.size(); i++) {
			Pair p = (Pair)pairs.get(i);
			seconds[i] = p.second;
		}
		return seconds;
	}
}
